"""Tests for our options submodule."""
import optparse
import os

import mock
import pytest

from flake8_polyfill import options


@pytest.mark.parametrize('value,expected', [
    ('E123,\n\tW234,\n    E206', ['E123', 'W234', 'E206']),
    ('E123,W234,E206', ['E123', 'W234', 'E206']),
    (['E123', 'W234', 'E206'], ['E123', 'W234', 'E206']),
    (['E123', '\n\tW234', '\n    E206'], ['E123', 'W234', 'E206']),
])
def test_parse_comma_separated_list(value, expected):
    """Verify that similar inputs produce identical outputs."""
    assert options.parse_comma_separated_list(value) == expected


@pytest.mark.parametrize('value,expected', [
    ('flake8', 'flake8'),
    ('../flake8', os.path.abspath('../flake8')),
    ('flake8/', os.path.abspath('flake8')),
])
def test_normalize_path(value, expected):
    """Verify that we normalize paths provided to the tool."""
    assert options.normalize_path(value) == expected


@pytest.mark.parametrize('value,expected', [
    ('file.py', 'file.py'),
    ('path/file.py', os.path.abspath('path/file.py')),
    (['file.py', 'path/file.py'],
     ['file.py', os.path.abspath('path/file.py')]),
])
def test_parse_normalized_paths(value, expected):
    """Verify that we handle strings and lists when normalizing paths."""
    assert options.parse_normalized_paths(value) == expected


@pytest.mark.parametrize(
    # NOTE: `defaults` has NO impact, since the callback being called implies
    # that a `value` was specified.
    'comma_separated_list, normalize_paths, defaults, value, expected_value', [
        (True, True, {}, 'val', 'N(C(val))'),
        (True, True, {'foo': 'defaultval'}, 'val', 'N(C(val))'),
        (True, False, {}, 'val', 'C(val)'),
        (True, False, {'foo': 'defaultval'}, 'val', 'C(val)'),
        (False, False, {}, 'val', 'val'),
        (False, False, {'foo': 'defaultval'}, 'val', 'val'),
    ]
)
def test_generate_callback_from_composition(
        comma_separated_list, normalize_paths, defaults,
        value, expected_value,
):
    """Verify our generate_callback_from composition.

    We mock out parse_comma_separated_list and parse_normalized_paths with
    simple string transformations for better readability.
    """
    dest = 'foo'
    opt_str = '--foo'
    option = optparse.Option(opt_str, dest=dest)
    parser = mock.Mock(values=optparse.Values(defaults))

    base_callback = mock.Mock()
    callback = options.generate_callback_from(
        comma_separated_list=comma_separated_list,
        normalize_paths=normalize_paths,
        base_callback=base_callback,
    )

    with mock.patch('flake8_polyfill.options.parse_comma_separated_list') as \
            parse_comma_separated_list, \
            mock.patch('flake8_polyfill.options.parse_normalized_paths') as \
            parse_normalized_paths:

        parse_comma_separated_list.side_effect = lambda v: 'C({})'.format(v)
        parse_normalized_paths.side_effect = lambda v: 'N({})'.format(v)
        callback(option, opt_str, value, parser)

    base_callback.assert_called_with(option, opt_str, expected_value, parser)


def test_store_callback():
    """Verify the default callback behaves like option with action='store'."""
    dest = 'foo'
    opt_str = '--foo'
    option = optparse.Option(opt_str, dest=dest)
    parser = mock.Mock(values=optparse.Values({'foo': 'defaultval'}))
    options.store_callback(option, opt_str, 'val', parser)
    assert parser.values.foo == 'val'


@pytest.fixture
def parser():
    """Provide a pycodestyle-esque OptionParser instance."""
    parser = optparse.OptionParser('flake8')
    parser.config_options = []
    return parser


def test_register_with_store_callback(parser):
    """Verify we handle typical no-custom-callback case (integration test)."""
    options.register(parser, '--foo', default=['path/file.py'], type='string',
                     comma_separated_list=True, normalize_paths=True)
    values, _ = parser.parse_args([])
    assert values.foo == ['path/file.py']  # default is used in its entirety
    values, _ = parser.parse_args(['--foo=file.py,path/file.py'])
    assert values.foo == ['file.py', os.path.abspath('path/file.py')]


def test_register_with_custom_callback(parser):
    """Verify we handle custom callback (integration test)."""
    def custom_callback(option, opt_str, value, parser, *args, **kwargs):
        parser.values.count = len(value)

    options.register(parser, '--foo', type='string', callback=custom_callback,
                     comma_separated_list=True, normalize_paths=True)
    values, _ = parser.parse_args(['--foo=file.py,path/file.py'])
    assert values.count == 2


def test_register_parse_from_config(parser):
    """Verify we append to config_options on registration."""
    options.register(parser, '--select', default='E123,W504',
                     parse_from_config=True)
    assert 'select' in parser.config_options



"""Tests for polyfill's stdin monkey patching."""
import pep8
import pycodestyle
import pytest

from flake8_polyfill import stdin
from flake8_polyfill import version


def test_modules_dict():
    """Verify that it works the way we expect."""
    assert len(stdin.modules['pep8']) == 1
    assert stdin.modules['pep8'] == [pep8]
    assert len(stdin.modules['pycodestyle']) == 1
    assert stdin.modules['pycodestyle'] == [pycodestyle]
    assert len(stdin.modules['all']) == 2
    assert stdin.modules['all'] == [pep8, pycodestyle]


@pytest.fixture
def pep8_mod(request):
    """Fixture to replace pep8's default stdin_get_value."""
    original_stdin_get_value = pep8.stdin_get_value

    def replace():
        pep8.stdin_get_value = original_stdin_get_value

    request.addfinalizer(replace)
    return pep8


@pytest.fixture
def pycodestyle_mod(request):
    """Fixture to replace pycodestyle's default stdin_get_value."""
    original_stdin_get_value = pycodestyle.stdin_get_value

    def replace():
        pycodestyle.stdin_get_value = original_stdin_get_value

    request.addfinalizer(replace)
    return pycodestyle


# NOTE(sigmavirus24): These are weak tests but *shrug* I'm not sure of a
# better way to test these.
def test_monkey_patch_all(pep8_mod, pycodestyle_mod):
    """Verify we monkey patch everything."""
    stdin.monkey_patch('all')
    assert pep8_mod.stdin_get_value is pycodestyle_mod.stdin_get_value


@pytest.mark.skipif(
    (2, 5) < version.version_info < (2, 7),
    reason='They are the same on everything less than 2.6.'
)
def test_monkey_patch_pep8(pep8_mod):
    """Verify we monkey patch pep8 only."""
    stdin.monkey_patch('pep8')
    assert pep8_mod.stdin_get_value is not pycodestyle.stdin_get_value


@pytest.mark.skipif(
    version.version_info < (2, 6),
    reason='They are the same on everything less than 2.6.'
)
def test_monkey_patch_pycodestyle(pycodestyle_mod):
    """Verify we monkey patch pycodestyle only."""
    stdin.monkey_patch('pycodestyle')
    assert pep8.stdin_get_value is not pycodestyle_mod.stdin_get_value


@pytest.mark.skipif(
    version.version_info < (3, 0) or version.version_info > (4, 0),
    reason='Requires Flake8 3.x'
)
def test_uses_flake8_util_stdin(pep8_mod, pycodestyle_mod):
    """Verify we monkey-patch using internal flake8 functions."""
    import flake8.utils

    stdin.monkey_patch('all')
    assert pep8_mod.stdin_get_value is flake8.utils.stdin_get_value
    assert pycodestyle_mod.stdin_get_value is flake8.utils.stdin_get_value



# -*- coding: utf-8 -*-
"""Packaging logic for Flake8's polyfill."""
import io
import os
import sys

import setuptools

sys.path.insert(0, os.path.join(os.path.dirname(__file__), 'src'))

import flake8_polyfill  # noqa

requires = ['flake8']


def get_long_description():
    """Generate a long description from the README file."""
    descr = []
    for fname in ('README.rst',):
        with io.open(fname, encoding='utf-8') as f:
            descr.append(f.read())
    return '\n\n'.join(descr)


setuptools.setup(
    name='flake8-polyfill',
    license='MIT',
    version=flake8_polyfill.__version__,
    description='Polyfill package for Flake8 plugins',
    long_description=get_long_description(),
    author='Ian Cordasco',
    author_email='graffatcolmingov@gmail.com',
    url='https://gitlab.com/pycqa/flake8-polyfill',
    package_dir={'': 'src'},
    packages=[
        'flake8_polyfill',
    ],
    install_requires=requires,
    classifiers=[
        "Environment :: Console",
        "Framework :: Flake8",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Software Development :: Quality Assurance",
    ],
)
